@@ -272,8 +272,11 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         return sorted(ep_ports) == sorted(spec_ports)
 
     def _has_pods(self, endpoints):
+        ep_subsets = endpoints.get('subsets', [])
+        if not ep_subsets:
+            return False
         return any(True
-                   for subset in endpoints.get('subsets', [])
+                   for subset in ep_subsets
                    for address in subset.get('addresses', [])
                    if address.get('targetRef', {}).get('kind') == 'Pod')
 
@@ -516,7 +519,8 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     project_id=lbaas_spec.project_id,
                     subnet_id=lbaas_spec.subnet_id,
                     ip=lbaas_spec.ip,
-                    security_groups_ids=lbaas_spec.security_groups_ids)
+                    security_groups_ids=lbaas_spec.security_groups_ids,
+                    service_type=lbaas_spec.type)
                 if lbaas_state.service_pub_ip_info is None:
                     service_pub_ip_info = (
                         self._drv_service_pub_ip.acquire_service_pub_ip_info(
